/* Keyboard routines                                                         */

/*--- Microsoft C 5.0 compiler include files                                 */

#include <math.h>
#include <stddef.h>
#include <stdio.h>

/*--- Blaise Computing C Tools Plus 5.00                                     */

#include <bfiles.h>
#include <bkeybrd.h>
#include <bkeys.h>
#include <bstrings.h>

/*--- other includes                                                         */

#include "data.h"
#include "driver.h"

#include "array.h"

#define KEYBOARD
#include "keyboard.h"

#include "screen.h"
#include "scrnfile.h"
#include "zplane.h"



/*---------------------------------------------------------------------------*/
/* Local variables                                                           */

int  OldChar;                          /* previous character                 */
int  OldCode;


/*---------------------------------------------------------------------------*/
/* Decode the current key and make something happen                          */
/* Called only when a new key is detected...                                 */
/* Returns 1 if key was used for something                                   */

int DecodeKey(void) {

char tempstr[_MAX_PATH];
unsigned int tempui,tempui1;
int ynresp;
DATUM temppoint;
int decoded;

  decoded = 1;                                   /* assume useful key        */

  switch (KeyChar) {

     case 0 :                                    /* extended keys            */
        switch (KeyCode) {
           case KB_S_N_F1 :                      /* show a little help       */
              SplitNew();
              printf("F2:Scrn F3:Dot F4:Img F5:Iter F6:Run\n"
                     " F7:Full F8:Load F9:Unzoom F10:Zoom");
              SplitScreen(1,FAST,INSTANT);
              SplitScreen(0,NORMAL,-INSTANT);
              break;

           case KB_S_A_F1 :                      /* show a little ID         */
              SplitNew();
              printf("Code by Ed Nisley\n");
              printf("Nisley Micro Consulting");
              SplitScreen(1,FAST,INSTANT);
              SplitScreen(0,NORMAL,READABLE);
              break;

           case KB_S_N_F2 :                      /* show screen location     */
              SplitNew();
              WhereScreen(&temppoint);
              printf("Screen at (%+12.10f,%+12.10f)\n"
                    "      size (%+13.5E,%+13.5E)",
                      temppoint.cplx.r,-temppoint.cplx.i,
                      ScreenSize.cplx.r,ScreenSize.cplx.i);
              SplitScreen(1,FAST,INSTANT);
              SplitScreen(0,POOF,-INSTANT);
              break;

           case KB_S_N_F3 :                      /* show pel location        */
              SplitNew();
              printf("Pel %2.0i (%+12.10f,%+12.10f)\n"
                    "        (%+13.5E,%+13.5E)",
                      ThisDot.scrn.color,
                      ThisDot.cplx.r,-ThisDot.cplx.i,
                      PelSize.r,PelSize.i);
              SplitScreen(1,FAST,INSTANT);
              SplitScreen(0,POOF,-INSTANT);
              break;

           case KB_S_N_F4 :                      /* show image location      */
              SplitNew();
              printf("Image at (%+12.10f,%+12.10f)\n"
                    " size     (%+13.5E,%+13.5E)",
                      ImageCenter.cplx.r,-ImageCenter.cplx.i,
                      ImageSize.cplx.r,ImageSize.cplx.i);
              SplitScreen(1,FAST,INSTANT);
              SplitScreen(0,POOF,-INSTANT);
              break;

           case KB_S_N_F5 :                      /* iteration limit, etc     */
              SplitNew();
              printf("Iteration limit: %u   New: ",
                      IterLimit);
              SplitScreen(1,FAST,INSTANT);
              tempui = GetUI(IterLimit);

              if (tempui != IterLimit) {
                 IterLimit = tempui;
                 ColorShift = (int) (0.5 + log((double)IterLimit)/log(2.0)) - 4;
                 ColorShift = max(ColorShift,0);

              }

              SplitNew();
              printf("Color shift: %u bits.  New: ",
                      ColorShift);
              tempui1 = GetUI(ColorShift);

              if (tempui1 != ColorShift) {
                 ColorShift = tempui1;
              }

              SplitNew();
              ynresp = GetYN("Clear buffer (y/n)? ");
              SplitScreen(0,POOF,INSTANT);

              if (tempui || tempui1) {
                 InitialCalc();
              }

              if (ynresp) {
                 SetupBuffer();                  /* clear the buffer         */
              }
              break;

           case KB_S_N_F6 :                      /* restart main loop        */
              DrawRun = 1;
              break;

           case KB_S_N_F7 :                      /* flip full/screen flag    */
              SplitNew();
              SplitScreen(1,FAST,INSTANT);
              ResetScroll();                     /* recenter screen          */

              if (GetYN("Full buffer mode (y/n)? ")) {
                 if (!ResultArea) {              /* change to full buffer    */
                    ResultArea = 1;
                    ResultCenter = ImageCenter;
                    ResultSize = ImageSize;
                    DrawRun = 0;
                 }
              }
              else {
                 if (ResultArea) {               /* change to screen only    */
                    ResultArea = 0;
                    ResultCenter = ImageCenter;
                    ResultSize = ScreenSize;
                    DrawRun = 0;
                 }
              }

              SplitNew();                        /* decide on bounce mode    */
              if (GetYN("Bounce mode (y/n)? ")) {
                 Vint_flags.hbounce = 1;
                 Vint_flags.hrun = 1;
                 Hpan.rate = 1;
                 Hpan.direction = -1;
                 Vint_flags.vbounce = 1;
                 Vint_flags.vrun = 1;
                 Vscroll.rate = 2;
                 Vscroll.direction = -1;
              }
              else {
                 Vint_flags.hbounce = 0;
                 Vint_flags.vbounce = 0;
              }
              SplitScreen(0,FAST,INSTANT);
              break;
           case KB_S_N_F8 :                      /* load a file              */
              SplitNew();
              printf("File to load: ");
              SplitScreen(1,FAST,INSTANT);
              if (!GetFSpec(tempstr)) {
                 ResetScroll();
                 LoadImage(tempstr);
              }
              SplitScreen(0,FAST,TEASER);
              break;

           case KB_S_N_F9 :                      /* zoom out                 */
              PopPlane();                        /*   this stops drawing     */
              break;

           case KB_S_N_F10 :                     /* zoom in                  */
              ZoomIn();
              break;

           case KB_S_N_RIGHT :                   /* window moves right       */
              if (Vint_flags.hrun == 0) {
                 Hpan.rate = HRATE;                        /* start motion   */
                 Hpan.direction = 1;
                 Vint_flags.hrun = 1;
              }
              else {                                       /* already moving */
                 if (Hpan.direction > 0) {
                    if (Hpan.rate > 1)
                       Hpan.rate >>= 1;                    /* faster right   */
                 }
                 else {
                    if (Hpan.direction < 0)
                       Vint_flags.hrun = 0;                /* stop left      */
                 }

              }
              break;

           case KB_S_N_LEFT :                    /* window moves left        */
              if (Vint_flags.hrun == 0) {
                 Hpan.rate = HRATE;                        /* start motion   */
                 Hpan.direction = -1;
                 Vint_flags.hrun = 1;
              }
              else {                                       /* already moving */
                 if (Hpan.direction < 0) {
                    if (Hpan.rate > 1)
                       Hpan.rate >>= 1;                    /* faster left    */
                 }
                 else {
                    if (Hpan.direction > 0)
                       Vint_flags.hrun = 0;                /* stop right     */
                 }

              }
              break;

           case KB_S_N_UP :                      /* window moves up          */
              if (Vint_flags.vrun == 0) {
                 Vscroll.rate = VRATE;                     /* start motion   */
                 Vscroll.direction = -1;
                 Vint_flags.vrun = 1;
              }
              else {                                       /* already moving */
                 if (Vscroll.direction < 0) {
                    if (Vscroll.rate > 1)
                       Vscroll.rate >>= 1;                 /* faster up      */
                 }
                 else {
                    if (Vscroll.direction > 0)
                       Vint_flags.vrun = 0;                /* stop down      */
                 }

              }
              break;

           case KB_S_N_DOWN :                    /* window moves down        */
              if (Vint_flags.vrun == 0) {
                 Vscroll.rate = VRATE;                     /* start motion   */
                 Vscroll.direction = 1;
                 Vint_flags.vrun = 1;
              }
              else {                                       /* already moving */
                 if (Vscroll.direction > 0) {
                    if (Vscroll.rate > 1)
                       Vscroll.rate >>= 1;                 /* faster down    */
                 }
                 else {
                    if (Vscroll.direction < 0)
                       Vint_flags.vrun = 0;                /* stop up        */
                 }

              }
              break;

           default :
              decoded = 0;                       /* key not used             */
              SplitNew();
              printf("Dead key\n scan code %i  ",KeyCode);

              SplitScreen(1,FAST,INSTANT);       /* show split screen        */
              SplitScreen(0,POOF,TEASER);        /* unsplit quickly          */
              break;
        }
        break;

     case KB_C_N_ESC :                           /* cancel and end           */
        DrawRun = 0;
        break;

     default :
        decoded = 0;                             /* key not used             */
        SplitNew();
        printf("Dead key\n scan code %i  ",KeyCode);
        if (isgraph(KeyChar))
           printf("glyph [%c]",(char)KeyChar);

        SplitScreen(1,FAST,INSTANT);             /* show split screen        */
        SplitScreen(0,POOF,TEASER);              /* unsplit quickly          */


  }

  return decoded;


} /* DecodeKey */


/*---------------------------------------------------------------------------*/
/* Get a new key                                                             */
/* Returns 1 if there's a key ready                                          */

int  GetKey(void) {

  char  keycharC;
  int   keycodeI;

  if (!kbready(&keycharC,&keycodeI))             /* check for key waiting    */
     return 0;

  OldChar = KeyChar;                             /* save old values          */
  OldCode = KeyCode;

  KeyChar = kbgetkey(&KeyCode);                  /* and get new ones...      */

  return 1;

} /* GetKey */

/*---------------------------------------------------------------------------*/
/* Get a yes/no response from keyboard, echoed to screen                     */
/* Requires "y" input to give nonzero return value                           */
/* Assumes that the split screen is ready to go                              */

int GetYN(char *prompt) {

char tempstr[10];
char *response;

   printf("%s",prompt);
   tempstr[0] = 5;
   response = cgets(tempstr);
   strlwr(response);

   return('y' == *response);

}

/*---------------------------------------------------------------------------*/
/* Get an unsigned integer from keyboard, echoed to screen                   */
/* Presumes that the split screen already displayed                          */

unsigned int GetUI(unsigned int defval) {

char response[100];                              /* must be very long        */
unsigned int temp;

  gets(response);

  stpcvt(response,RWHITE);                       /* discard whitespace       */

  if (0 != strlen(response)) {
     temp = (unsigned int) atol(response);       /* do half-assed conversion */
  }
  else {
     temp = defval;
  }

  return temp;

}

/*---------------------------------------------------------------------------*/
/* Get a file spec from the keyboard, echoed to screen                       */
/* Presumes that the split screen already displayed with some text...        */
/* Input is pointer to buffer for file spec                                  */

int GetFSpec(char *fspec) {

char response[_MAX_PATH];
int  fnameloc;

  gets(response);                                /* get the raw text         */

  stpcvt(response,RWHITE);                       /* discard whitespace       */

  if (flnorm(response,fspec,&fnameloc)) {        /* normalize filespec       */
     return 1;                                   /*   ... no good            */
  }
  else {
     return 0;                                   /*   ... OK                 */
  }


}

